﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Collections;
using System.Xml;
using System.IO;
using System.Text;

using OSGeo.MapGuide;


/// <summary>
/// Summary description for Utility. Created by Daniel Du, DevTech
/// </summary>
public class Utility
{

    MgSiteConnection siteConnection;

    public void InitializeWebTier(HttpRequest Request)
    {
        string realPath = Request.ServerVariables["APPL_PHYSICAL_PATH"];
        String configPath = realPath + "webconfig.ini";
        MapGuideApi.MgInitializeWebTier(configPath);
    }

    public void ConnectToServer(String sessionID)
    {
        MgUserInformation userInfo = new MgUserInformation(sessionID);
        siteConnection = new MgSiteConnection();
        siteConnection.Open(userInfo);
    }



    public Dictionary<string, string> GetAllMaps()
    {
        MgResourceIdentifier rootResId = new MgResourceIdentifier("Library://");
        return GetAllMaps(rootResId);
    }

    /// <summary>
    ///  enumberate all the map definitions in the 
    /// </summary>
    /// <param name="resId"></param>
    /// <returns></returns>
    public Dictionary<string, string> GetAllMaps(MgResourceIdentifier resId)
    {
        Dictionary<string, string> mapList = new Dictionary<string, string>();

        MgResourceService resSvc = siteConnection.CreateService(MgServiceType.ResourceService) as MgResourceService;

        // Get all map def resources
        MgByteReader byteRdr = resSvc.EnumerateResources(resId, -1, MgResourceType.MapDefinition);

        //Convert to string 
        String MapResStr = byteRdr.ToString();

        //Load into XML document so we can parse and get the names of the maps
        XmlDocument doc = new XmlDocument();
        doc.LoadXml(MapResStr);

        //let's extract the map names and list them        
        XmlNodeList MapResIdNodeList;
        XmlElement root = doc.DocumentElement;
        MapResIdNodeList = root.SelectNodes("//ResourceId");
        int nRes = MapResIdNodeList.Count;

        for (int i = 0; i < nRes; i++)
        {
            XmlNode MapResIdNode = MapResIdNodeList.Item(i);
            String MapRes = MapResIdNode.InnerText;
            int index1 = MapRes.LastIndexOf('/') + 1;
            int index2 = MapRes.IndexOf("MapDefinition") - 2;
            int length = index2 - index1 + 1;
            string mapName = MapRes.Substring(index1, length);
            if (!mapList.ContainsKey(mapName))
            {
                mapList.Add(mapName, MapRes);
            }

        }

        return mapList;


    }

    /// <summary>
    /// Edit the weblayout resource content to change map
    /// </summary>
    /// <param name="webLayout">the resourece id of weblayout</param>
    /// <param name="sessionId">current session string</param>
    /// <param name="mapDefResourceId">resource id of map definition</param>
    /// <returns></returns>
    public string SwitchMapsInWebLayout(string webLayout, string sessionId, string mapDefResourceId)
    {
        MgResourceIdentifier layoutResId = new MgResourceIdentifier(webLayout);
        MgResourceService resSvc = siteConnection.CreateService(MgServiceType.ResourceService) as MgResourceService;
        //the resource content of WebLayout
        MgByteReader reader = resSvc.GetResourceContent(layoutResId);
        string layoutXml = reader.ToString();

        //Edit the map resource in XmlDocument in DOM
        XmlDocument doc = new XmlDocument();

        doc.LoadXml(layoutXml);
        XmlNode nodeCenterX = doc.GetElementsByTagName("ResourceId").Item(0);
        nodeCenterX.InnerText = mapDefResourceId;

        MgByteSource byteSource = ByteSourceFromXMLDoc(doc);
        string sessionLayoutName = layoutResId.GetName();
        string sessionLayout = "Session:" + sessionId + @"//" + sessionLayoutName + ".WebLayout";
        MgResourceIdentifier sessionLayoutResId = new MgResourceIdentifier(sessionLayout);

        //update back to a session weblayout
        resSvc.SetResource(sessionLayoutResId, byteSource.GetReader(), null);

        return sessionLayout;


    }


    public MgByteSource ByteSourceFromXMLDoc(XmlDocument xmlDoc)
    {
        //Save the amended DOM object to a memory stream
        MemoryStream XmlStream = new MemoryStream();
        xmlDoc.Save(XmlStream);

        //Now get the memory stream into a byte array that can be read into an MgByteSource
        byte[] byteNewDef = XmlStream.ToArray();
        String sNewDef = new String(Encoding.UTF8.GetChars(byteNewDef));
        byte[] byteNewDef2 = new byte[byteNewDef.Length - 1];
        int iNewByteCount = Encoding.UTF8.GetBytes(sNewDef, 1, sNewDef.Length - 1, byteNewDef2, 0);
        MgByteSource byteSource = new MgByteSource(byteNewDef2, byteNewDef2.Length);
        byteSource.SetMimeType(MgMimeType.Xml);
        return byteSource;
    }

    public static string GetStringFromMemoryStream(MemoryStream m)
    {
        if (m == null || m.Length == 0)
            return null;

        m.Flush();
        m.Position = 0;
        StreamReader sr = new StreamReader(m);
        string s = sr.ReadToEnd();

        return s;
    }

    public static MemoryStream GetMemoryStreamFromString(string s)
    {
        if (s == null || s.Length == 0)
            return null;

        MemoryStream m = new MemoryStream();
        StreamWriter sw = new StreamWriter(m);
        sw.Write(s);
        sw.Flush();

        return m;
    }


    public string ChangeInitialViewInWebLayout(string webLayoutTemplate, string sessionId, string centerX, string centerY, string scale)
    {
        if (siteConnection == null)
        {
            MgUserInformation userInfo = new MgUserInformation(sessionId);
            siteConnection = new MgSiteConnection();
            siteConnection.Open(userInfo);
        }

        MgResourceIdentifier layoutResId = new MgResourceIdentifier(webLayoutTemplate);
        MgResourceService resSvc = siteConnection.CreateService(MgServiceType.ResourceService) as MgResourceService;
        MgByteReader reader = resSvc.GetResourceContent(layoutResId);

        //string layoutXml = reader.ToString();  // empty here for unknown reason, using following workaround
        System.IO.MemoryStream ms = new System.IO.MemoryStream();
        byte[] buf = new byte[8 * 1024];
        int read = 1;
        while (read != 0)
        {
            read = reader.Read(buf, buf.Length);
            ms.Write(buf, 0, read);
        }

        string layoutXml = GetStringFromMemoryStream(ms);

        XmlDocument doc = new XmlDocument();

        doc.LoadXml(layoutXml);

        // if using custom view in web layout defintion, change the custom view port
        if (doc.GetElementsByTagName("InitialView").Count > 0)
        {
            XmlNode nodeCenterX = doc.GetElementsByTagName("CenterX").Item(0);
            nodeCenterX.InnerText = centerX;
            XmlNode nodeCenterY = doc.GetElementsByTagName("CenterY").Item(0);
            nodeCenterY.InnerText = centerY;
            XmlNode scaleNode = doc.GetElementsByTagName("Scale").Item(0);
            scaleNode.InnerText = scale;
        }
        else // using the map's initial view, we need to add a custom view port ourselves.
        {
            //Add <InitialView> tag for the web layout(application definition)

            XmlNode initialViewNode = doc.CreateNode(XmlNodeType.Element, "InitialView", null);
            XmlNode centerXNode = doc.CreateElement("CenterX");
            centerXNode.InnerText = centerX;
            XmlNode centerYNode = doc.CreateElement("CenterY");
            centerYNode.InnerText = centerY;
            XmlNode scaleNode = doc.CreateElement("Scale");
            scaleNode.InnerText = scale;

            initialViewNode.AppendChild(centerXNode);
            initialViewNode.AppendChild(centerYNode);
            initialViewNode.AppendChild(scaleNode);

            //insert before the <Map> tag
            doc.GetElementsByTagName("MapGroup")[0].InsertBefore(initialViewNode, doc.GetElementsByTagName("Map")[0]);
        }

        MgByteSource byteSource = ByteSourceFromXMLDoc(doc);
        string sessionLayoutName = layoutResId.GetName();
        string sessionLayout = "Session:" + sessionId + @"//" + sessionLayoutName + ".ApplicationDefinition";
        MgResourceIdentifier sessionLayoutResId = new MgResourceIdentifier(sessionLayout);

        resSvc.SetResource(sessionLayoutResId, byteSource.GetReader(), null);

        return sessionLayout;

    }
}
